import 'package:buttons_swiper/src/swiperController.dart';
import 'package:card_swiper/card_swiper.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class swiper extends StatelessWidget {
  final swiperControll = Get.put(SwiperGetxController());
  final Alignment alignement;
  final double height;
  final Color circleBorderColor;
  double circleBorderWidth;
  final ValueChanged<int> onTap;
  final ValueChanged<int>? onSwip;
  List<Widget> firstIconsList;
  List<Widget> secondIconsList;
  bool loop;
  double fade;
  int startIndex;
  double viewportFraction;
  double scale;
  List<Widget>? pages;

  swiper({
    required this.alignement,
    required this.height,
    this.circleBorderWidth = 5,
    required this.circleBorderColor,
    required this.onTap,
    this.onSwip,
    required this.firstIconsList,
    required this.secondIconsList,
    this.loop = false,
    this.fade = 0.01,
    this.startIndex = 0,
    this.viewportFraction = 0.3,
    this.scale = 0.01,
    this.pages,
  }) {
    swiperControll.borderWidth.value = this.circleBorderWidth;
    swiperControll.index = this.startIndex;
    if (firstIconsList.length != secondIconsList.length) {
      throw "Please Provide Equal Icon Arrays";
    }
    if (pages != null) {
      if (firstIconsList.length != pages!.length) {
        throw "Please Provide Equal Icons And Pages";
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        pages != null
            ? pageView(
          pages: pages!,
        )
            : Container(),
        Align(
          alignment: alignement,
          child: Container(
            height: height,
            child: Stack(
              children: [
                Obx(
                      () => Center(
                    child: AnimatedContainer(
                      width: swiperControll.width.value,
                      height: swiperControll.hight.value,
                      decoration: BoxDecoration(
                          border: Border.all(
                            color: circleBorderColor,
                            width: swiperControll.borderWidth.value,
                          ),
                          shape: BoxShape.circle),
                      duration: Duration(milliseconds: 100),
                      curve: Curves.bounceIn,
                    ),
                  ),
                ),
                Swiper(
                  controller: swiperControll.controller,
                  itemBuilder: (BuildContext context, int index) {
                    return Animated_Container(
                      pages: pages,
                      onTap: (Index) async {
                        if (index == swiperControll.index) {
                          await swiperControll.animate(Index);
                        }
                        onTap.call(Index);
                      },
                      index: index,
                      icon: firstIconsList[index],
                      secondIcon: secondIconsList[index],
                    );
                  },
                  itemCount: firstIconsList.length,
                  onIndexChanged: (index) async {
                    swiperControll.index = index;
                    if (onSwip != null && !swiperControll.inProgress) {
                      onSwip!.call(index);
                    }
                    if (pages != null) {
                      swiperControll.page_controller.animateToPage(index,
                          duration: Duration(milliseconds: 300), curve: Curves.ease);
                    }
                  },
                  loop: loop,
                  fade: fade,
                  index: startIndex,
                  pagination: SwiperPagination(
                      alignment: Alignment.bottomCenter,
                      builder: SwiperPagination.rect),
                  viewportFraction: viewportFraction,
                  scale: scale,
                ),
              ],
            ),
          ),
        ),
      ],
    );
  }
}

class Animated_Container extends StatefulWidget {
  Widget icon;
  Widget secondIcon;
  final ValueChanged<int> onTap;
  final int index;
  List<Widget>? pages;

  Animated_Container({
    required this.icon,
    required this.secondIcon,
    required this.onTap,
    required this.index,
    this.pages,
  }) {}

  @override
  _AnimatedContainerState createState() => _AnimatedContainerState(
    onTap: onTap,
    icon: icon,
    secondIcon: secondIcon,
    index: index,
    pages: pages,
  );
}

/////////////////////////////////////////////////////////////
class _AnimatedContainerState extends State<Animated_Container>
    with SingleTickerProviderStateMixin {
  Widget icon;
  Widget secondIcon;
  late Widget shownIcon;
  final ValueChanged<int> onTap;
  final int index;
  late AnimationController animationController;
  final swiperControll = Get.find<SwiperGetxController>();
  List<Widget>? pages;

  _AnimatedContainerState({
    required this.icon,
    required this.secondIcon,
    required this.onTap,
    required this.index,
    this.pages,
  }) {
    shownIcon = icon;
  }

  @override
  void initState() {
    animationController = new AnimationController(
      vsync: this,
    );
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return InkWell(
      borderRadius: BorderRadius.circular(360),
      child: AnimatedSwitcher(
        transitionBuilder: (child, animation) => ScaleTransition(
          scale: animation,
          child: child,
        ),
        child: shownIcon,
        duration: Duration(milliseconds: 500),
        reverseDuration: Duration(milliseconds: 250),
      ),
      onTap: () async {
        animate();
        swiperControll.inProgress = true;
        await swiperControll.controller.move(index);
        if (pages != null) {
          await swiperControll.page_controller.animateToPage(index,
              duration: Duration(milliseconds: 300), curve: Curves.ease);
        }
        swiperControll.inProgress = false;
        onTap.call(index);
      },
    );
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  void animate() async {
    setState(() {
      shownIcon = secondIcon;
    });
    await Future.delayed(const Duration(milliseconds: 500), () {});
    setState(() {
      shownIcon = icon;
    });
  }
}

class pageView extends StatelessWidget {
  List<Widget> pages;
  final swiperControll = Get.find<SwiperGetxController>();
  late PageController controller;

  pageView({
    required this.pages,
  }) {}

  @override
  Widget build(BuildContext context) {
    return PageView(
      controller: swiperControll.page_controller,
      children: pages,
      onPageChanged: (index) async {
        swiperControll.inProgress = true;
        await swiperControll.controller.move(index);
        swiperControll.inProgress = false;
      },
    );
  }
}
